/* cgolwin.cpp
 *  - implementation of gameoflifeWin
 *  - render does most of the important stuff
 *   - event takes the X events
 *
 *  (c) 2003 Andrew Chant
 *  licensed under GPL
 *
 *   This file is part of cgol.
    cgol is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    cgol is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with cgol; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <cairo.h>
#include <math.h>
#include <iostream>
#include <assert.h>
#include "cgolwin.h"
using std::cout;

gameoflifeWin::gameoflifeWin(Ctkapp * app, int wi, int h, int r, int c, bool s)
 :Ctkwin(app,wi,h)
{
	rows = r;
	cols = c;
	width = wi;
	height = h;
	use_cairo_show_surface = s;
	game = new gameoflife(rows, cols);

	cr = cairo_create();
	
	XGCValues xgcv;
	// Helps prevent *some* choking on opaque resize 
	xgcv.graphics_exposures = false;
	
	gc = XCreateGC(dpy, w, GCGraphicsExposures, &xgcv);
	XSetForeground(dpy, gc, WhitePixel(dpy, DefaultScreen(dpy)));
	// MAX 24 fps, really just means 1/24 s between advancements
	app->regtimer(0,1000000/24,-1,advanceTimer, (void *)this);
	buffer = grid = 0;
}

void gameoflifeWin::event(XEvent * e)
{
	switch (e->type)
	{
	case Expose:
		if (((XExposeEvent *)e)->count == 0) // NOT RELIABLE :(
		{
			if (!XPending(dpy)) //This is an ugly hack
					render();
		}
		break;
	case CreateNotify:
		width = ((XCreateWindowEvent *)e)->width;
		height = ((XCreateWindowEvent *)e)->height;
		if (grid != 0)
		{
			XFreePixmap(dpy, grid);
			XFreePixmap(dpy, buffer);
			buffer = grid = 0;
		}
		break;
	case ConfigureNotify:
		width = ((XConfigureEvent *)e)->width;
		height = ((XConfigureEvent *)e)->height;
		if (grid != 0)
		{
			XFreePixmap(dpy, grid);
			XFreePixmap(dpy, buffer);
			buffer = grid = 0;
		}
		break;
	default:
		//cout << "Unhandled Event " << e->type << "\n";
		;
	}
}

void gameoflifeWin::advanceTimer(void * obj)
{
	gameoflifeWin * game = (gameoflifeWin *)obj;
	game->game->advance(); // yeah ok bad naming shoot me
	game->render();
}

void gameoflifeWin::render()
{ 	/* note: all cairo ops operate on a rowxcol coordinate system
	   	 the cairo transform matrix handles resizing etc */
	if (grid == 0)
	{	  
		cairo_default_matrix(cr);
		cairo_scale(cr, (double)width/(double)cols,(double)height/(double)rows); 
		cairo_set_line_width(cr, 0.1); // set to one tenth of a unit... will that be row or col?
		radius = 0.5; // so, the edges of circles in connecting cells should just touch.
		
		buffer = XCreatePixmap(dpy, w, width, height, DefaultDepth(dpy, DefaultScreen(dpy))); // cairo co-ords
		/* draw the grid into a pixmap which can be copied over the buffer each cycle */
		grid = XCreatePixmap(dpy, w, width, height, DefaultDepth(dpy, DefaultScreen(dpy))); // these are in pixels, not
		XFillRectangle(dpy, grid, gc, 0, 0, width, height);
		
		if (use_cairo_show_surface) // Broken
		{	//todo : fix mem leak here 
			cairo_save(cr);
			cairo_default_matrix(cr);
			cairo_scale(cr,50,50);
			cellpix = XCreatePixmap(dpy, w, 50, 
					50, DefaultDepth(dpy, DefaultScreen(dpy)));
			XFillRectangle(dpy, cellpix, gc, 0, 0, 50, 50);
			surface = cairo_xlib_surface_create(dpy, cellpix, 
					DefaultVisual(dpy, DefaultScreen(dpy)),
					CAIRO_FORMAT_RGB24, DefaultColormap(dpy, DefaultScreen(dpy)));
			if (surface == NULL)
				exit(1); // no mem
			cairo_set_target_surface(cr, surface);
			cairo_set_rgb_color(cr,0,0,0);
			cairo_arc(cr, .5,  .5, .5, 0, 2*M_PI);
			cairo_fill(cr);
			cairo_restore(cr);
		}
		
		
		cairo_set_target_drawable(cr, dpy, grid);
		cairo_set_rgb_color(cr, .8, .8, .8);
		for (int col = 0; col <= cols; col++)
		{
			cairo_move_to(cr, col, 0);
			cairo_rel_line_to(cr, 0, rows);
		}
		for (int row = 0; row <= rows; row++)
		{
			cairo_move_to(cr, 0, row);
			cairo_rel_line_to(cr, cols, 0);
		}
		cairo_stroke(cr);
		cairo_set_rgb_color(cr, 0, 0, 0);
	}
	
	cells = game->getboard();
	// Copy gen'd grid to buffer
	XCopyArea(dpy, grid, buffer, gc, 0, 0, width, height, 0, 0);
	cairo_set_target_drawable(cr, dpy, buffer);
	for (list<struct cell>::const_iterator i = cells->begin();
		i != cells->end(); i++)
	{
		double red,green,blue;
		red = green = blue = 0;
		//Note colour weighting avoids white (Max .75 of each)
		if (i->dna & B1) blue += .50;
		if (i->dna & B2) blue += .25;
		if (i->dna & R1) red += .50;
		if (i->dna & R2) red += .25;
		if (i->dna & G1) green += .50;
		if (i->dna & G2) green += .25;
		
		if (use_cairo_show_surface)
		{
			//Note: broken 
			cairo_move_to(cr, i->x, i->y);
			cairo_show_surface(cr, surface, 1, 1);
		}
		else 
		{ 
			cairo_set_rgb_color(cr, red, green, blue);
			cairo_arc(cr, i->x + .5, i->y + .5, radius, 0, 2*M_PI);
			cairo_fill(cr);
		}
	}
	cairo_set_rgb_color(cr, 0, 0, 0);
	XCopyArea(dpy, buffer, w, gc, 0, 0, width, height, 0, 0);
	XFlush(dpy);
}
