/*
 * Copyright  2003 USC, Information Sciences Institute
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the name of the
 * University of Southern California not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission. The University of Southern
 * California makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without express
 * or implied warranty.
 *
 * THE UNIVERSITY OF SOUTHERN CALIFORNIA DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE UNIVERSITY OF
 * SOUTHERN CALIFORNIA BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Carl D. Worth <cworth@isi.edu>
 */

#include <cairo.h>
#include <math.h>

#include "write_png.h"

#define TEXT "hello, world"

#define WIDTH 450
#define HEIGHT 900
#define STRIDE (WIDTH * 4)

char image[STRIDE*HEIGHT];
int
main (void)
{
    int i;
    cairo_t *cr;
    cairo_matrix_t *matrix;
    double angle;

    cr = cairo_create ();

    cairo_set_target_image (cr, image, CAIRO_FORMAT_ARGB32,
			    WIDTH, HEIGHT, STRIDE);

    cairo_set_rgb_color (cr, 0., 0., 0.);

    cairo_select_font (cr, "serif", 0, 0);
    cairo_scale_font (cr, 40);
    cairo_translate (cr, 40, 40);

#define NUM_STRINGS 3.0
    cairo_select_font (cr, "mono", 0, 0);

    cairo_scale_font (cr, 12);
    cairo_show_text (cr, "+CTM rotation");

    cairo_save (cr);
    cairo_select_font (cr, "serif", 0, 0);
    cairo_scale_font (cr, 40);
    for (i=0; i < NUM_STRINGS; i++) {
	angle = (double) i * 0.5 * M_PI / (NUM_STRINGS - 1);
	cairo_save (cr);
	cairo_rotate (cr, angle);
	cairo_move_to (cr, 100, 0);
	cairo_show_text (cr, "Text");
	cairo_restore (cr);
    }
    cairo_restore (cr);

    cairo_translate (cr, 0, HEIGHT / 3);

    cairo_move_to (cr, 0, 0);
    cairo_show_text (cr, "+CTM rotation");
    cairo_rel_move_to (cr, 0, 12);
    cairo_show_text (cr, "-font rotation");

    cairo_save (cr);
    cairo_select_font (cr, "serif", 0, 0);
    cairo_scale_font (cr, 40);
    for (i=0; i < NUM_STRINGS; i++) {
	angle = (double) i * 0.5 * M_PI / (NUM_STRINGS - 1);
	cairo_save (cr);
	cairo_rotate (cr, angle);
	matrix = cairo_matrix_create ();
	cairo_matrix_rotate (matrix, - angle);
	cairo_transform_font (cr, matrix);
	cairo_matrix_destroy (matrix);
	cairo_move_to (cr, 100, 0);
	cairo_show_text (cr, "Text");
	cairo_restore (cr);
    }
    cairo_restore (cr);

    cairo_translate (cr, 0, HEIGHT / 3);

    cairo_move_to (cr, 0, 0);
    cairo_show_text (cr, "+CTM rotation");
    cairo_rel_move_to (cr, 0, 12);
    cairo_show_text (cr, "-CTM rotation");

    cairo_save (cr);
    cairo_select_font (cr, "serif", 0, 0);
    cairo_scale_font (cr, 40);
    for (i=0; i < NUM_STRINGS; i++) {
	angle = (double) i * 0.5 * M_PI / (NUM_STRINGS - 1);
	cairo_save (cr);
	cairo_rotate (cr, angle);
	cairo_move_to (cr, 100, 0);
	cairo_rotate (cr, -angle);
	cairo_show_text (cr, "Text");
	cairo_restore (cr);
    }
    cairo_restore (cr);

    write_png_argb32 (image, "text-rotate.png", WIDTH, HEIGHT, STRIDE);

    cairo_destroy (cr);

    return 0;
}
