#!/usr/bin/env python

import os, sys, re, time
import cgi, string, sys, cStringIO
import keyword, token, tokenize
import HTMLParser
import gtk

_KEYWORD = token.NT_OFFSET + 1
_TEXT    = token.NT_OFFSET + 2

css_classes = {
        token.NUMBER     : "py_number",
        token.OP         : "py_operator",
        token.STRING     : "py_string",
        tokenize.COMMENT : "py_comment",
        token.NAME       : "py_name",
        token.ERRORTOKEN : "py_error_token",
        _KEYWORD         : "py_keyword",
        _TEXT            : "py_text",
}

actions = {
        "date" : time.asctime(),
}

macros = {
        "gnome"                   : "http://www.gnome.org",
        "gtk"                     : "http://www.gtk.org",
        "atk"                     : "http://www.gtk.org",
        "gdk"                     : "http://www.gtk.org",
        "glib"                    : "http://www.gtk.org",
        "python"                  : "http://www.python.org",
        "pygtk"                   : "http://www.pygtk.org",
        "glade"                   : "http://glade.gnome.org",
        "libglade"                : "http://developer.gnome.org/doc/API/libglade/libglade.html",
        "diveintopython"          : "http://www.diveintopython.org",
        "gimp"                    : "http://www.gimp.org",
        "pango"                   : "http://www.pango.org",
        "utf"                     : "http://www.unicode.org",
        "unicode"                 : "http://www.unicode.org",
        "sax"                     : "http://www.sax.org/",
        "pygtkmoz"                : "http://sourceforge.net/projects/pygtkmoz",
        "gpl"                     : "http://www.gnu.org/licenses/gpl.html",
        "lgpl"                    : "http://www.gnu.org/licenses/lgpl.html",
        "fdl"                     : "http://www.gnu.org/licenses/fdl.html",
        "mandrake"                : "http://www.linux-mandrake.com",
		"cairo"					  :	"http://cairographics.org",
		"freedesktop"			  : "http://freedesktop.org",
        "simplegladeapp.py"       : "simplegladeapp.html",
        "simple-glade-codegen.py" : "simple-glade-codegen.html",
        "tigrux-tutor.py"         : "tigrux-tutor.html",
}

class Parser:
        def __init__(self, raw):
                self.raw = raw.expandtabs().strip()
                self.content = []

        def __str__(self):
                self.lines = [0, 0]
                pos = 0
                while 1:
                        pos = self.raw.find("\n", pos) + 1
                        if not pos:
                                break
                        self.lines.append(pos)
                self.lines.append(len(self.raw))
                self.pos = 0
                text = cStringIO.StringIO(self.raw)
                try:
                        tokenize.tokenize(text.readline, self)
                except tokenize.TokenError, ex:
                        msg = ex[0]
                        line = ex[1][0]
                        error = "<h3>ERROR: %s</h3>%s\n" % (msg, self.raw[self.lines[line]:])
                        self.content.append(error)
                return "".join(self.content)

        def __call__(self, toktype, toktext, (srow,scol), (erow,ecol), line):
                oldpos = self.pos
                newpos = self.lines[srow] + scol
                self.pos = newpos + len(toktext)

                if toktype in [token.NEWLINE, tokenize.NL]:
                        self.content.append("\n")
                        return

                if newpos > oldpos:
                        self.content.append(self.raw[oldpos:newpos])

                if toktype in [token.INDENT, token.DEDENT]:
                        self.pos = newpos
                        return

                if token.LPAR <= toktype and toktype <= token.OP:
                        toktype = token.OP
                elif toktype == token.NAME and keyword.iskeyword(toktext):
                        toktype = _KEYWORD
                css_class = css_classes.get(toktype, css_classes[_TEXT])

                self.content.append('<span class="%s">' % css_class)
                self.content.append(cgi.escape(toktext))
                self.content.append("</span>")

class MenuNode:
        def __init__(self, level=0, anchor="", label=""):
                self.level = level
                self.anchor = anchor
                self.label = label
                self.childNodes = []

        def push(self, node):
                if isinstance(node, MenuNode):
                        diff = node.level - self.level
                        if diff == 1:
                                self.childNodes.append(node)
                        elif diff > 1:
                                if self.childNodes:
                                        self.childNodes[-1].push(node)
                                else:
                                        print 'Warning: Could not add node "%s"' % (node.label)
                else:
                        raise TypeError, "Invalid argument"

        def __str__(self):
                html_list = []
                if self.level != 0:
                        html_data = '%s<li><a href="#%s">%s</a></li>\n'
                        data =  html_data % (self.level * "\t", self.anchor, self.label)
                        html_list.append(data)
                if self.childNodes:
                        data = '%s<ul class="menu">\n' % (self.level * "\t")
                        html_list.append(data)
                        for node in self.childNodes:
                                html_list.append( str(node) )
                        data = '%s</ul>\n' % (self.level * "\t")
                        html_list.append(data)
                return "".join(html_list)

class IndexGenerator(HTMLParser.HTMLParser):
        def __init__(self):
                HTMLParser.HTMLParser.__init__(self)
                self.header_re = re.compile("h(\d)")
                self.last_anchor = None
                self.last_anchor_index = 0
                self.last_header_index = 0
                self.headers = []
                self.anchors = []
                self.while_header = 0
                self.menu = MenuNode()
                self.node_index = 0

        def handle_starttag(self, name, attrs):
                attrs = dict(attrs)
                self.node_index += 1
                match = self.header_re.match(name)
                if match:
                        self.last_header_index = self.node_index
                        self.while_header = int(match.groups()[0])
                        return
                if name == "a":
                        if "name" in attrs:
                                self.last_anchor_index = self.node_index
                                self.last_anchor = attrs["name"]
                        return

        def handle_data(self, content):
                content = content.strip()
                if self.while_header and self.last_anchor:
                        if self.last_header_index - self.last_anchor_index < 5:
                                node = MenuNode(self.while_header, self.last_anchor, content)
                                self.menu.push(node)

        def handle_endtag(self, name):
                match = self.header_re.match(name)
                if match:
                        self.while_header = 0

def not_found(f):
        print "Warning: Could not find file %s" % f

def include_source_action(match):
        include_source = match.groups()[0]
        include_source_base = os.path.split(include_source)[1]
        include_source_anchor = "_".join( re.findall("[a-zA-z01-9_]+", include_source_base) )
        if os.path.isfile(include_source):
                include_source_content = open(include_source).read()
                content = Parser(include_source_content)
                anchor = '''\
<a name="%s" href="%s">
        <h3>%s</h3>
</a>
<pre>%s</pre>
'''
                return anchor % (include_source_anchor,include_source,include_source_base,content)
        else:
                not_found(include_glade)
                return include_source

def include_glade_action(match):
        include_glade = match.groups()[0]
        include_glade_base = os.path.split(include_glade)[1]
        include_glade_anchor = "_".join( re.findall("[a-zA-z01-9_]+", include_glade_base) )
        if os.path.isfile(include_glade):
                anchor = '''\
<a name="%s" href="%s">
        <h3>%s</h3>
</a>
'''
                return anchor % (include_glade_anchor,include_glade,include_glade_base)
        else:
                not_found(include_glade)
                return include_glade


def include_screenshot_action(match):
        screenshot = match.groups()[0]
        screenshot_base = os.path.split(screenshot)[1]
        screenshot_ext = os.path.splitext(screenshot_base)[1][1:]
        screenshot_anchor = "_".join( re.findall("[a-zA-z01-9_]+", screenshot_base) )
        if os.path.isfile(screenshot):
                screenshot_pixbuf = gtk.gdk.pixbuf_new_from_file(screenshot)
                screenshot_width  = screenshot_pixbuf.get_width()
                screenshot_height = screenshot_pixbuf.get_height()
                thumbnail_height = screenshot_pixbuf.get_height()
                if thumbnail_height > 160:
                        thumbnail_height = 160
                thumbnail_width  = thumbnail_height*screenshot_width/screenshot_height
                thumbnail = "%s-thumb%s" % os.path.splitext(screenshot)
                if not os.path.isfile(thumbnail):
                        thumbnail_pixbuf = screenshot_pixbuf.scale_simple(
                                                                        thumbnail_width, thumbnail_height,
                                                                        gtk.gdk.INTERP_BILINEAR)
                        thumbnail_pixbuf.save(thumbnail, screenshot_ext)
                html = '''\
<div class="screenshot">
        <a href="%(screenshot)s" name="%(base)s">
                <img src="%(thumbnail)s" alt="%(base)s" width="%(width)s" height="%(height)s" />
                <br />
                <h3>%(base)s</h3>
        </a>
</div>
'''
                d = {}
                d["screenshot"] = screenshot
                d["base"] = screenshot_base
                d["thumbnail"] = thumbnail
                d["width"] = thumbnail_width
                d["height"] = thumbnail_height
                return html % d
        else:
                not_found(screenshot)
                return screenshot

def macro_action(match):
        macro = match.groups()[0]
        macro_lower = macro.lower()
        if macro_lower in macros:
                url = macros[macro_lower]
                anchor = '<a href="%s">%s</a>' % (url,macro)
                return anchor
        elif macro_lower in actions:
                return actions[macro_lower]
        else:
                print "Warning: Could not expand macro %s" % macro_lower

def http_action(match):
        url = match.groups()[0]
        anchor = '<a href="http://%s">http://%s</a>' % (url,url)
        return anchor

def ftp_action(match):
        url = match.groups()[0]
        anchor = '<a href="ftp://%s">ftp://%s</a>' % (url,url)
        return anchor

def file_action(match):
        url = match.groups()[0]
        tail =  os.path.split(url)[1]
        anchor = '<a href="%s">%s</a>' % (url,tail)
        return anchor

def include_index_action(match):
        global source
        index_generator = IndexGenerator()
        index_generator.feed(source)
        return str(index_generator.menu)

def main(input_file, output_file):
        global source
        source = open(input_file).read()

        valid_chars = r"(\.|\w|\d|_|-|/|~)+"
        at = "@"
        valid_chars_at = (at,valid_chars)
        at_valid_chars_at = (at,valid_chars, at)

        http_re_s = r"%shttp:\/\/(?P<url>%s)" % valid_chars_at
        http_re = re.compile(http_re_s)
        source = http_re.sub(http_action, source)

        ftp_re_s = r"%sftp:\/\/(?P<url>%s)" % valid_chars_at
        ftp_re = re.compile(ftp_re_s)
        source = ftp_re.sub(ftp_action, source)

        file_re_s = r"%sfile:(?P<url>%s)" % valid_chars_at
        file_re = re.compile(file_re_s)
        source = file_re.sub(file_action, source)

        include_source_re_s = r"%sinclude_source\((?P<source>%s)\)" % valid_chars_at
        include_source_re = re.compile(include_source_re_s)
        source = include_source_re.sub(include_source_action, source)

        include_glade_re_s = r"%sinclude_glade\((?P<source>%s)\)" % valid_chars_at
        include_glade_re = re.compile(include_glade_re_s)
        source = include_glade_re.sub(include_glade_action, source)

        include_screenshot_re_s = r"%sinclude_screenshot\((?P<source>%s)\)" % valid_chars_at
        include_screenshot_re = re.compile(include_screenshot_re_s)
        source = include_screenshot_re.sub(include_screenshot_action, source)

        macro_re_s = r"%s(?P<macro>%s)%s" % at_valid_chars_at
        macro_re = re.compile(macro_re_s)
        source = macro_re.sub(macro_action, source)

        index_re_s = r"%sinclude_index" % at
        index_re = re.compile(index_re_s)
        source = index_re.sub(include_index_action, source)

        open(output_file, "wt").write(source)

def usage():
        print "Usage %s <file>.tutor.html"

if __name__ == "__main__":
        if len(sys.argv) < 2:

                sys.exit(1)
        base_tutor , html_ext = os.path.splitext(sys.argv[1])
        if html_ext != ".html":
                usage()
                sys.exit(1)
        base, tutor_ext = os.path.splitext(base_tutor)
        if tutor_ext != ".tutor":
                usage()
                sys.exit(1)
        input_file = "%s%s%s" % (base, tutor_ext, html_ext)
        output_file = "%s%s" % (base, html_ext)
        if not os.path.isfile(input_file):
                print "File %s does not exist"
                sys.exit(1)
        print "Parsing %s" % input_file
        main(input_file, output_file)
        print "Created %s" % output_file
